package main

import (
	"fmt"
	"golang.org/x/crypto/ssh"
	"io"
	"log"
	"net"
	"os"
)

var user string
var knownHostKey ssh.PublicKey
var localAddr string
var remoteAddr string
var sshHost string
var signer ssh.Signer

func main() {
	var keyBuffer []byte
	keyBuffer = []byte(os.Getenv("SSHWAY_PRIVATE_KEY"))
	if len(keyBuffer) == 0 {
		log.Fatalf("SSHWAY_PRIVATE_KEY is empty")
	}

	// Create the Signer for this private key.
	var err error
	signer, err = ssh.ParsePrivateKey(keyBuffer)
	if err != nil {
		log.Fatalf("Unable to parse private key: %v", err)
	}

	user = os.Getenv("SSHWAY_USER")
	if len(user) == 0 {
		log.Fatalf("SSHWAY_USER is empty")
	}

	knownHostBuffer := []byte(os.Getenv("SSHWAY_KNOWN_HOST"))
	if len(knownHostBuffer) == 0 {
		log.Fatalf("SSHWAY_KNOWN_HOST is empty")
	}

	knownHostKey, _, _, _, err = ssh.ParseAuthorizedKey(knownHostBuffer)
	if err != nil {
		log.Fatalf("Unable to parse known host key %v: %v", knownHostBuffer, err)
	}

	localAddr = os.Getenv("SSHWAY_LOCAL")
	if len(localAddr) == 0 {
		log.Fatalf("SSHWAY_LOCAL is empty")
	}

	remoteAddr = os.Getenv("SSHWAY_REMOTE")
	if len(remoteAddr) == 0 {
		log.Fatalf("SSHWAY_REMOTE is empty")
	}

	sshHost = os.Getenv("SSHWAY_SSH_HOST")
	if len(sshHost) == 0 {
		log.Fatalf("SSHWAY_SSH_HOST is empty")
	}

	server, err := net.Listen("tcp", localAddr)
	if err != nil {
		log.Fatal("Error listening:", err.Error())
	}
	fmt.Printf("Listening on %s\n", localAddr)
	defer server.Close()

	client_counter := 0

	for {
		connection, err := server.Accept()
		if err != nil {
			log.Fatalf("Error accepting local client: %v\n", err)
		}
		log.Printf("%6d New connection from client: %s\n", client_counter, connection.RemoteAddr().String())
		go handleClient(client_counter, connection)
		client_counter++
	}
}

func handleClient(client_counter int, localSock net.Conn) {
	defer localSock.Close()

	config := &ssh.ClientConfig{
		User: user,
		Auth: []ssh.AuthMethod{
			// Use the PublicKeys method for remote authentication.
			ssh.PublicKeys(signer),
		},
		HostKeyCallback: ssh.FixedHostKey(knownHostKey),
		// HostKeyAlgorithms: []string{"ssh-ed25519"},
	}

	conn, err := ssh.Dial("tcp", sshHost, config)
	if err != nil {
		log.Printf("%6d SSH connection failed: %v\n", client_counter, err)
		return
	}

	forwardClient, err := conn.Dial("tcp", remoteAddr)
	if err != nil {
		log.Printf("%6d Port forward failed: %v\n", client_counter, err)
		conn.Close()
		return
	}

	go func() {
		io.Copy(forwardClient, localSock)
		log.Printf("%6d Local client disconnected\n", client_counter)
		conn.Close()
	}()

	io.Copy(localSock, forwardClient)
	log.Printf("%6d Remote socket closed for local client\n", client_counter)
	conn.Close()
}
